import { getOrderBySubOrderNumber, saveOrder } from '@/api/order'
import { OrderData, ShipmentForm, ShipmentOrderRes } from '@/types/api/order'
import { useValue } from '@/utils/hooks/useValue'
import { showError } from '@/utils/ui'
import { ref } from 'vue'
const [orderList, resetOrderList] = useValue<OrderData[]>([])
const [shipmentForm, resetShipmentForm] = useValue<ShipmentForm>({
  shippingWay: '',
  carriageName: '',
  logisticsTracking: '',
  carriageAmount: '',
  namespace: '',
})
export default function useShipment(callback?: () => void) {
  const shipmentFormRef = ref()
  const productionOrderNumber = ref('')
  const shipmentVisible = ref(false)
  const isLock = ref(false)
  const inputRef = ref()
  const orderNumber = ref('')
  const shipmentLoading = ref(false)
  const searchShipmentByOrderNumber = async () => {
    const code = productionOrderNumber.value
    shipmentVisible.value = true
    if (!code) {
      isLock.value = false
      inputRef.value.focus()
      return ElMessage({
        message: '请录入生产单号',
        type: 'warning',
        offset: window.innerHeight / 2,
      })
    }
    let rowData
    for (const item of orderList.value) {
      rowData = item.productList?.find((jj) => jj.subOrderNumber === code)
      if (rowData) break
    }
    if (rowData) {
      orderNumber.value = code
      const unShipmentNum = (rowData.num || 0) - (rowData.shipmentNum || 0)
      if (unShipmentNum > (rowData.count || 0)) {
        rowData.count || rowData.count === 0
          ? (rowData.count += 1)
          : (rowData.count = 0)
      } else {
        ElMessage({
          message: '拣货数不能大于未发数',
          type: 'warning',
          offset: window.innerHeight / 2,
        })
      }
      productionOrderNumber.value = ''
      isLock.value = false
      inputRef.value.focus()
    } else {
      getPackingData(code)
    }
  }
  const getPackingData = async (code: string) => {
    try {
      shipmentLoading.value = true
      const res = await getOrderBySubOrderNumber(code)
      if (res.data) {
        for (const item of res.data.productList || []) {
          if (item.num) {
            item.count = 0
          }
          if (item.subOrderNumber === code) {
            item.count || item.count === 0
              ? (item.count += 1)
              : (item.count = 0)
          }
        }
        const index = orderList.value.findIndex(
          (item) => item.id === res.data.id,
        )
        if (index === -1) {
          orderList.value.unshift(res.data)
        }
      }
      orderNumber.value = code
      productionOrderNumber.value = ''
      isLock.value = false
      inputRef.value.focus()
    } catch (e) {
      productionOrderNumber.value = ''
      isLock.value = false
      inputRef.value.focus()
      showError(e)
    } finally {
      shipmentLoading.value = false
    }
  }
  const saveShipment = async () => {
    if (orderList.value.length === 0) {
      return ElMessage({
        message: '请选择订单',
        type: 'warning',
        offset: window.innerHeight / 2,
      })
    }
    try {
      await shipmentFormRef.value?.validate()
    } catch {
      return
    }
  
    const isEqual = orderList.value.some(
      (item) => item.namespace === orderList.value[0].namespace,
    )
    if (!isEqual) {
      ElMessage({
        message: '请选择同一工厂的订单',
        type: 'warning',
        offset: window.innerHeight / 2,
      })
      return
    }
    const data: ShipmentOrderRes[] = []
    orderList.value.forEach((item) => {
      shipmentForm.value.namespace = item.namespace
      const order = []
      for (const jj of item.productList || []) {
        if (jj.count === 0) continue
        const subOrder = {
          erpSubOrderNumber: jj.erpSubOrderNumber,
          subOrderNumber: jj.subOrderNumber,
          sendOutQuantity: jj.count,
          version: jj.version
        }
        order.push(subOrder)
      }
      if (order) {
        data.push(...order)
      }
    })
    try {
      const res = await saveOrder(data, shipmentForm.value)
      ElMessage.success(res.message)
      shipmentVisible.value = false
      callback && callback()
    } catch (e) {
      showError(e)
    }
  }
  const confirmDelivery = async () => {
    shipmentVisible.value = true
    productionOrderNumber.value = ''
    resetOrderList()
    resetShipmentForm()
  }
  const onShipmentDialogOpened = () => {
    inputRef.value?.focus()
    shipmentFormRef.value?.clearValidate()
  }
  return {
    shipmentFormRef,
    productionOrderNumber,
    shipmentVisible,
    inputRef,
    shipmentForm,
    orderNumber,
    shipmentLoading,
    orderList,
    searchShipmentByOrderNumber,
    saveShipment,
    confirmDelivery,
    onShipmentDialogOpened
  }
}
